import pytest  # noqa


def test_busday_locator_import():
    from busdayaxis._locator import BusdayLocator

    locator = BusdayLocator()
    assert locator is not None


def test_busday_locator_with_axis():
    import matplotlib.pyplot as plt

    from busdayaxis._locator import BusdayLocator

    fig, ax = plt.subplots()
    locator = BusdayLocator()
    locator.set_axis(ax.xaxis)

    plt.close(fig)



# Copyright (c) Simon Niederberger.
# Distributed under the terms of the Modified BSD License.

import matplotlib.dates as mdates
import matplotlib.scale as mscale
import matplotlib.transforms as mtransforms
import numpy as np
import pandas as pd
from pandas.core.dtypes.common import is_nested_list_like

from busdayaxis._locator import BusdayLocator

EPOCH = np.datetime64("1970-01-01", "D")


def _datetime_to_busday_float(values, **busday_kwargs) -> np.ndarray:
    """Convert datetime-like values to business-day floats."""

    values = pd.DatetimeIndex(pd.to_datetime(values))

    day = values.floor("D")
    intraday = (values - day).total_seconds() / 86400.0

    weekday = values.weekday
    rolled = day + pd.to_timedelta(
        np.where(weekday == 5, 2, np.where(weekday == 6, 1, 0)), unit="D"
    )

    business_days = np.busday_count(
        EPOCH, rolled.values.astype("datetime64[D]"), **busday_kwargs
    ).astype(float)

    intraday = np.where(weekday >= 5, 0.0, intraday)

    return business_days + intraday


def _busday_float_to_datetime(values, **busday_kwargs) -> pd.DatetimeIndex:
    """Convert business-day floats back to datetime-like values."""

    values = np.asarray(values, dtype=float)

    day_int = np.floor(values).astype(int)
    date = np.busday_offset(EPOCH, day_int, roll="forward", **busday_kwargs).astype(
        "datetime64[s]"
    )

    intraday_frac = values - day_int
    intraday_seconds = (
        (intraday_frac * 86400.0).astype("int64").astype("timedelta64[s]")
    )

    datetime = date + intraday_seconds

    return pd.to_datetime(datetime)


class BusdayTransform(mtransforms.Transform):
    """Transform for business time.

    Forward transform (float days -> business float)
    """

    input_dims = 1
    output_dims = 1
    is_separable = True
    _busday_kwargs = {}

    def __init__(self, **busday_kwargs):
        self._busday_kwargs = busday_kwargs
        super().__init__()

    def transform_non_affine(self, x):
        """
        Convert matplotlib date numbers (days since 1970-01-01) to business-day floats.

        Rules:
            - Monday..Friday increase linearly
            - Fractional part represents fraction of 24h
            - Saturday/Sunday collapse to Monday 00:00
        """
        dates = mdates.num2date(x)

        if is_nested_list_like(dates):
            return [_datetime_to_busday_float(d, **self._busday_kwargs) for d in dates]
        else:
            return _datetime_to_busday_float(dates, **self._busday_kwargs)

    def inverted(self):
        return InvertedBusdayTransform(**self._busday_kwargs)


class InvertedBusdayTransform(mtransforms.Transform):
    """Inverse transform (business float -> matplotlib date numbers)"""

    input_dims = 1
    output_dims = 1
    is_separable = True
    _busday_kwargs = {}

    def __init__(self, **busday_kwargs):
        self._busday_kwargs = busday_kwargs
        super().__init__()

    def transform_non_affine(self, x):
        x = np.atleast_1d(np.asarray(x, dtype=float))
        dates = _busday_float_to_datetime(x, **self._busday_kwargs)
        result = mdates.date2num(dates)
        return result

    def inverted(self):
        return BusdayTransform(**self._busday_kwargs)


class BusdayScale(mscale.ScaleBase):
    name = "busday"
    _busday_kwargs = {}

    def __init__(self, axis, **kwargs):
        self._busday_kwargs = kwargs.copy()
        super().__init__(axis)

    def get_transform(self):
        return BusdayTransform(**self._busday_kwargs)

    def set_default_locators_and_formatters(self, axis):
        axis._busday_kwargs = self._busday_kwargs.copy()
        majloc = BusdayLocator()
        majloc.set_axis(axis)
        majfmt = mdates.DateFormatter("%Y-%m-%d")
        axis.set_major_locator(majloc)
        axis.set_major_formatter(majfmt)

    def limit_range_for_scale(self, vmin, vmax, minpos):
        return vmin, vmax


def register_scale():
    mscale.register_scale(BusdayScale)



"""
Hourly random walk with zero weekend returns.

Calendar axis:
    - Daily ticks
    - Weekend shading
    - Flat weekend segments

Business axis:
    - Weekends compressed
"""

# %%
import matplotlib.dates as mdates
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

import busdayaxis

busdayaxis.register_scale()

num_days = 20
dates = pd.date_range("2025-01-01", periods=num_days * 24, freq="h")

returns = np.random.normal(0, 0.002, len(dates))
returns[dates.weekday >= 5] = 0.0

prices = 100 * (1 + pd.Series(returns, index=dates)).cumprod()


fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(8, 7), sharey=True)

fig.suptitle(
    "Comparison of scale='linear' vs scale='busday'", fontsize=14, fontweight="bold"
)


# --- Calendar axis ---
ax1.plot(dates, prices.values, linewidth=1.3)
ax1.set_title("Calendar Time (scale='linear')")
ax1.set_ylabel("Price")

ax1.xaxis.set_major_locator(mdates.DayLocator())
ax1.xaxis.set_major_formatter(mdates.DateFormatter("%Y-%m-%d"))
ax1.tick_params(axis="x", rotation=90)

# Shade weekends
full_days = pd.date_range(dates.min().normalize(), dates.max().normalize(), freq="D")

for d in full_days:
    if d.weekday() == 5:  # Saturday
        ax1.axvspan(d, d + pd.Timedelta(days=2), color="grey", alpha=0.15, linewidth=0)


# --- Business axis ---
ax2.plot(dates, prices.values, linewidth=1.3)
ax2.set_xscale("busday")
ax2.set_title("Business Time (scale='busday')")
ax2.set_ylabel("Price")
ax2.tick_params(axis="x", rotation=90)

# Mark weekend boundaries
for d in full_days:
    if d.weekday() == 5:
        ax2.axvline(d, linestyle="--", linewidth=0.8, alpha=0.6)
        ax2.axvline(d + pd.Timedelta(days=2), linestyle="--", linewidth=0.8, alpha=0.6)


plt.tight_layout(rect=[0, 0, 1, 0.96])
plt.show()

# %%



"""
Business Day Scale Example
==========================

This example demonstrates how to use the business day scale in matplotlib.
"""

# %%
import matplotlib.pyplot as plt
import pandas as pd

import busdayaxis

busdayaxis.register_scale()

dates = pd.date_range("2026-01-01", periods=25, freq="D")
values = range(len(dates))


# %%
# Linear Scale Example
fig, ax = plt.subplots()

ax.plot(dates, values)
ax.set_xscale("linear")
ax.tick_params(axis="x", rotation=90)

plt.show()

# %%
# Business Day Scale Example
fig, ax = plt.subplots()

ax.plot(dates, values)
ax.set_xscale("busday")
ax.tick_params(axis="x", rotation=90)

plt.show()

# %%
# Business Day Scale Example
fig, ax = plt.subplots()

ax.plot(dates, values)
ax.set_xscale(busdayaxis.BusdayScale(ax))
ax.tick_params(axis="x", rotation=90)

plt.show()

# %%
# Business Day Scale Example with holidays
fig, ax = plt.subplots()

ax.plot(dates, values)
ax.set_xscale(busdayaxis.BusdayScale(ax, holidays=["2026-01-01", "2026-01-08"]))
ax.tick_params(axis="x", rotation=90)
plt.show()

# %%
# Business Day Scale Example with holidays
fig, ax = plt.subplots()

ax.plot(dates, values)
ax.set_xscale("busday", holidays=["2026-01-01", "2026-01-08"])
ax.tick_params(axis="x", rotation=90)
plt.show()



# Copyright (c) Simon Niederberger.
# Distributed under the terms of the Modified BSD License.


from ._locator import BusdayLocator
from ._scale import BusdayScale, register_scale

__all__ = ["BusdayScale", "BusdayLocator", "register_scale"]



# Copyright (c) Simon Niederberger.
# Distributed under the terms of the Modified BSD License.

import matplotlib.dates as mdates
import matplotlib.ticker as mticker
import numpy as np


class BusdayLocator(mticker.Locator):
    """
    Wrap an arbitrary datetime locator and remove
    ticks that fall on non-business days.

    Parameters
    ----------
    base_locator : matplotlib.ticker.Locator
        Any datetime-based locator (e.g. AutoDateLocator)
    busday_kwargs :
        Passed to np.is_busday (e.g. weekmask, holidays)
    """

    def __init__(self, base_locator=None):
        if base_locator is None:
            base_locator = mticker.MultipleLocator(1)
        self.base_locator = base_locator

    def set_axis(self, axis):
        super().set_axis(axis)
        self.base_locator.set_axis(axis)

    def __call__(self):
        ticks = self.base_locator()

        if len(ticks) == 0:
            return ticks

        busday_kwargs = getattr(self.axis, "_busday_kwargs", {})

        dts = mdates.num2date(ticks)
        days = np.array([np.datetime64(dt.replace(tzinfo=None), "D") for dt in dts])
        mask = np.is_busday(days, **busday_kwargs)
        return np.asarray(ticks)[mask]



import pytest  # noqa


def test_import():
    import busdayaxis

    assert hasattr(busdayaxis, "BusdayScale")
    assert hasattr(busdayaxis, "BusdayLocator")


def test_busday_scale_registration():
    import busdayaxis as busdayaxis

    busdayaxis.register_scale()


def test_busday_scale_with_holidays():
    import matplotlib.pyplot as plt

    holidays = ["2024-01-01", "2024-12-25"]

    fig, ax = plt.subplots()
    ax.set_xscale("busday", holidays=holidays)

    plt.close(fig)


def test_plot_dates():
    import matplotlib.pyplot as plt
    import pandas as pd

    dates = pd.date_range("2024-01-01", periods=10, freq="D")
    values = range(len(dates))

    fig, ax = plt.subplots()
    ax.plot(dates, values)
    ax.set_xscale("busday")

    plt.close(fig)
